﻿/*  Clear Rubble Command
    Copyright (C) 2018 "Spoffy" - https://www.github.com/Spoffy

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.*/

using System;
using System.Linq;
using System.Reflection;
using Eco.Core.Utils;
using Eco.Gameplay.Objects;
using Eco.Gameplay.Players;
using Eco.Gameplay.Systems.Chat;
using Eco.Mods.TechTree;

namespace Eco.Mods
{
    public class ClearRubbleCommand : IChatCommandHandler
    {
        private static Type RubbleNameToType(string rubbleName)
        {
            switch (rubbleName.ToLower())
            {
                case "stone":
                    return typeof(RubbleObject<StoneItem>);
                case "copper":
                    return typeof(RubbleObject<CopperOreItem>);
                case "gold":
                    return typeof(RubbleObject<GoldOreItem>);
                case "coal":
                    return typeof(RubbleObject<CoalItem>);
                case "iron":
                    return typeof(RubbleObject<IronOreItem>);
            }
            return null;
        }
        
        [ChatCommand("Clears all of the stone rubble from the server. Valid rubble types are 'stone', 'copper', 'gold', 'coal', 'iron'", ChatAuthorizationLevel.Admin)]
        public static void ClearRubble(User user, string rubbleName = "")
        {
            if (rubbleName.Length == 0)
            {
                ChatManager.ServerMessageToPlayer("You need to specify an item type", user);
                return;
            }
            
            var objectsField = typeof(EcoObjectManager).GetField("objects", BindingFlags.NonPublic | BindingFlags.Instance);
            if (objectsField == null)
            {
                ChatManager.ServerMessageToPlayer("Failed to grab objects. Report this to the mod developer.", user);
                return;
            }

            ThreadSafeHashSet<IEcoObject> objects = (ThreadSafeHashSet<IEcoObject>) objectsField.GetValue(EcoObjectManager.Obj);
            if (objects == null)
            {
                ChatManager.ServerMessageToPlayer("Unable to access list of objects. Report this to the mod developer.", user);
                return;
            }

            Type rubbleType = RubbleNameToType(rubbleName);
            if (rubbleType == null)
            {
                ChatManager.ServerMessageToPlayer("That is not a valid rubble type.", user);
                return;
            }

            var rubbleObjects = objects.Where(ecoObject => rubbleType.IsInstanceOfType(ecoObject)).ToList();
            ChatManager.ServerMessageToPlayer("Found " + rubbleObjects.Count + " of that type of rubble.", user);

            MethodInfo destroyRubble = rubbleType.GetMethod("Destroy", BindingFlags.NonPublic | BindingFlags.Instance);
            if (destroyRubble == null)
            {
                ChatManager.ServerMessageToPlayer("Unable to determine how to destroy that rubble. Report this to the mod developer.", user);
                return;
            }
            rubbleObjects.ForEach((ecoObject) =>
            {
                RubbleObject rubble = (RubbleObject) ecoObject;
                destroyRubble.Invoke(rubble, new object[0]);
            });
        }
    }
}